﻿// **********************************************
// Dies ist eines der 322 Rezepte aus dem
// C# 2005 Premium Codebook. 
// Das Snippet wurde vom Autor mit freundlicher 
// Genehmigung von Addison Wesley 
// bei dotnet-snippets.de veröffentlicht.
// **********************************************


using System;
using System.IO;
using System.Text;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using System.Collections.Generic;

/// <summary>
/// Repräsentiert eine MCI-Ausnahme
/// </summary>
public class MciException : Exception
{
    public MciException(string message)
    {
    }
}

/// <summary>
/// Klasse für das Abspielen von Mediadateien über MCI 
/// </summary>
public class Mci : IDisposable
{
    /* Deklaration der MCI-Funktionen, -Konstanten und -Strukturen */
    [DllImport("winmm.dll", CharSet = CharSet.Auto)]
    private static extern int mciSendString(string lpstrCommand,
       StringBuilder lpstrReturnString, int uReturnLength, IntPtr
       hwndCallback);

    [DllImport("winmm.dll", CharSet = CharSet.Auto)]
    private static extern int mciGetErrorString(int dwError, StringBuilder
       lpstrBuffer, int uLength);

    [DllImport("kernel32.dll", CharSet = CharSet.Auto)]
    private static extern int GetShortPathName(string lpszLongPath,
       StringBuilder lpszShortPath, int cchBuffer);

    /* Eigenschaft für den MCI-Alias */
    private List<string> m_AliasList;

    public Mci()
    {
        m_AliasList = new List<string>();
    }

    public bool IsOpen(string alias)
    {
        return m_AliasList.Contains(alias);
    }

    /// <summary>
    /// Liefert eine Fehlermeldung zu einem gegebenen 
    /// Fehlercode
    /// </summary>
    /// <param name="errorCode">Der Fehlercode</param>
    /// <returns>Gibt die Fehlermeldung zurück</returns>
    private string GetMciError(int errorCode)
    {
        StringBuilder errorMessage = new StringBuilder(255);
        if (mciGetErrorString(errorCode, errorMessage,
           errorMessage.Capacity) == 0)
        {
            return "MCI-Fehler " + errorCode;
        }
        else
        {
            return errorMessage.ToString();
        }
    }

    /// <summary>
    /// Öffnet eine Multimediadatei
    /// </summary>
    /// <param name="filename">Der Dateiname</param>
    public void Open(string filename, string alias)
    {
        Open(filename, alias, null);
    }

    /// <summary>
    /// Öffnet eine Multimediadatei
    /// </summary>
    /// <param name="filename">Der Dateiname</param>
    /// <param name="owner">Der Besitzer der erzeugten MCI-Instanz</param>
    public void Open(string filename, string alias, Control owner)
    {
        if (IsOpen(alias))
            return;

        // Überprüfen, ob die Datei existiert
        if (File.Exists(filename) == false)
        {
            throw new FileNotFoundException("Die Datei '" + filename +
               "' existiert nicht", filename);
        }

        // Den kurzen Dateinamen ermitteln
        StringBuilder shortPath = new StringBuilder(261);
        if (GetShortPathName(filename, shortPath, shortPath.Capacity) == 0)
        {
            throw new MciException("Fehler beim Auslesen des kurzen " +
               "Dateinamens für '" + filename + "': Windows-Fehler " +
               Marshal.GetLastWin32Error());
        }

        // MCI-Befehlsstring zum Öffnen zusammensetzen
        string mciString = "open " + shortPath.ToString() +
           " type mpegvideo alias " + alias;

        if (owner != null)
        {
            mciString += " parent " + (int)owner.Handle + " style child";
        }

        // MCI-Gerät öffnen
        int result = mciSendString(mciString, null, 0, IntPtr.Zero);
        if (result != 0)
        {
            throw new MciException("Fehler beim Öffnen des MCI-Geräts: " +
               GetMciError(result));
        }

        // Alias für das MCI-Gerät erzeugen
        m_AliasList.Add(alias);

        // Das Zeitformat für Längen- und Positionsangaben explizit
        // auf Millisekunden setzen
        mciString = "set " + alias + " time format ms";
        result = mciSendString(mciString, null, 0, IntPtr.Zero);
        if (result != 0)
        {
            throw new MciException("Fehler beim Setzen des Zeitformats: " +
               GetMciError(result));
        }
    }

    public int getLength(string alias)
    {
        StringBuilder buffer = new StringBuilder(261);
            int result = mciSendString("status " + alias + " length",
               buffer, buffer.Capacity, IntPtr.Zero);
            if (result != 0)
            {
                throw new MciException("Fehler beim Lesen von 'Length': " +
                   GetMciError(result));
            }

            return int.Parse(buffer.ToString());
    }

    /// <summary>
    /// Spielt eine geladene Multimediadatei ab
    /// </summary>
    /// <param name="repeat">Gibt an, ob das Abspielen nach dem Ende wieder von vorne beginnen soll</param>
    public void Play(string alias, bool repeat)
    {
        Play(alias, 0, getLength(alias), repeat);
    }

    /// <summary>
    /// Spielt eine geladene Multimediadatei ab einer bestimmten Position ab
    /// </summary>
    /// <param name="from">Die Startposition</param>
    /// <param name="repeat">Gibt an, ob das Abspielen nach dem Ende wieder von vorne beginnen soll</param>
    public void Play(string alias, int from, bool repeat)
    {
        Play(alias, from, getLength(alias) - from, repeat);
    }

    /// <summary>
    /// Spielt eine geladene Multimediadatei ab einer bestimmten Position 
    /// und bis zu einer bestimmten anderen Position ab
    /// </summary>
    /// <param name="from">Die Startposition</param>
    /// <param name="to">Die Endposition</param>
    /// <param name="repeat">Gibt an, ob das Abspielen nach dem Ende wieder von vorne beginnen soll</param>
    public void Play(string alias, int from, int to, bool repeat)
    {
        string mciString = "play " + alias +
           " from " + from + " to " + to;
        if (repeat)
        {
            mciString += " repeat";
        }
        int result = mciSendString(mciString, null, 0, IntPtr.Zero);
        if (result != 0)
        {
            throw new MciException("Fehler beim Aufruf von 'Play': " +
               GetMciError(result));
        }
    }

    public void setVolume(string alias, int volume)
    {
        int result = mciSendString("setaudio " + alias +
              " volume to " + volume, null, 0, IntPtr.Zero);
        if (result != 0)
        {
            throw new MciException("Fehler beim Aufruf von 'SetAudio': " +
               GetMciError(result));
        }
    }

    public void setBalance(string alias, int balance)
    {
        int result = mciSendString("setaudio " + alias + " left volume to " + (1000 - balance).ToString(), null, 0, IntPtr.Zero);

        if (result != 0)
        {
            throw new MciException("Fehler beim Aufruf von 'SetAudio': " +
               GetMciError(result));
        }

        result = mciSendString("setaudio " + alias + " right volume to " + balance.ToString(), null, 0, IntPtr.Zero);

        if (result != 0)
        {
            throw new MciException("Fehler beim Aufruf von 'SetAudio': " +
               GetMciError(result));
        }
    }

    /// <summary>
    /// Schließt ein aktuell geöffnetes MCI-Gerät
    /// </summary>
    public void Close()
    {
        m_AliasList.ForEach(alias => close(alias));
    }

    private void close(string alias)
    {
        if (IsOpen(alias))
        {
            m_AliasList.Remove(alias);
            string mciString = "close " + alias;
            int result = mciSendString(mciString, null, 0, IntPtr.Zero);
            if (result != 0)
                MessageBox.Show(GetMciError(result));
        }
    }

    /// <summary>
    /// Schließt ein aktuell geöffnetes MCI-Gerät
    /// </summary>
    public void Dispose()
    {
        this.Close();
    }

    /// <summary>
    /// Finalisierer
    /// </summary>
    ~Mci()
    {
        this.Close();
    }
}
